//
// C++ Interface: kbtaskview
//
// Description: 
//
//
// Author: Magnus Kulke <mkulke@magnusmachine>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KBTASKVIEW_H
#define KBTASKVIEW_H

#include <tdelistview.h>

/**
@author Magnus Kulke
*/
class KbTaskView : public TDEListView
{
TQ_OBJECT
public:
    KbTaskView(TQWidget *parent = 0, const char *name = 0);
    ~KbTaskView();
	 TQListViewItem* LastChild();
public slots:
	void SLOT_DoubleClicked(TQListViewItem* item, const TQPoint& p, int c);
	//virtual bool acceptDrag(TQDropEvent* event) const;
	//virtual TQDragObject* dragObject();
};

#endif
