//
// C++ Interface: KbItem
//
// Description: 
//
//
// Author: mkulke <sikor_sxe@radicalapproach.de>, (C) 2003
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KBITEM_H
#define KBITEM_H

#include <tqlistview.h>

class KbFileInfo;

/**
@author mkulke
*/
class KbItem : public TQListViewItem
{
public:
	enum entrytype
	{
		dir = 1001,
		file
	};
	KbItem(KbFileInfo kfi, TQListView* parent, TQListViewItem* after);
	KbItem(TQListView* parent, TQListViewItem* after);
	int compare (TQListViewItem * i, int col, bool ascending) const;
	~KbItem();
	
	TQString File() { return m_file; };
	TQString Path() { return m_path; };
	TQString Date() { return m_date; };
	unsigned int DateInt() { return m_date_int; };
	off64_t Size() { return m_size; };
		
protected:
	TQString m_path;
	TQString m_date;
	off64_t m_size;
	unsigned int m_date_int;
	TQString m_file;

};

#endif
