//
// C++ Interface: kbbookmarkitem
//
// Description: 
//
//
// Author: Magnus Kulke <mkulke@magnusmachine>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KBBOOKMARKITEM_H
#define KBBOOKMARKITEM_H

#include <tqlistview.h>

class KbSiteInfo;

/**
@author Magnus Kulke
*/
class KbBookmarkItem : public TQListViewItem
{
public:
	KbBookmarkItem(TQListView *view, TQListViewItem *after, KbSiteInfo *site);
	~KbBookmarkItem();
	KbSiteInfo* GetSiteInfo() { return mp_siteinfo; };
private: 
	KbSiteInfo* mp_siteinfo;

};

#endif
