// This file is generated by tdeconfig_compiler from kbconfig.kcfg.
// All changes you do to this file will be lost.

#include "kbconfig.h"

#include <kstaticdeleter.h>

KbConfig *KbConfig::mSelf = 0;
static KStaticDeleter<KbConfig> staticKbConfigDeleter;

KbConfig *KbConfig::self()
{
  if ( !mSelf ) {
    staticKbConfigDeleter.setObject( mSelf, new KbConfig() );
    mSelf->readConfig();
  }

  return mSelf;
}

KbConfig::KbConfig(  )
  : TDEConfigSkeleton( TQString::fromLatin1( "kasablancarc" ) )
{
  mSelf = this;
  setCurrentGroup( TQString::fromLatin1( "general" ) );

  TDEConfigSkeleton::ItemString  *itemSkiplist;
  itemSkiplist = new TDEConfigSkeleton::ItemString( currentGroup(), TQString::fromLatin1( "Skiplist" ), mSkiplist );
  addItem( itemSkiplist, TQString::fromLatin1( "Skiplist" ) );
  TDEConfigSkeleton::ItemString  *itemPrioritylist;
  itemPrioritylist = new TDEConfigSkeleton::ItemString( currentGroup(), TQString::fromLatin1( "Prioritylist" ), mPrioritylist );
  addItem( itemPrioritylist, TQString::fromLatin1( "Prioritylist" ) );
  TDEConfigSkeleton::ItemBool  *itemDirCachingIsEnabled;
  itemDirCachingIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "DirCachingIsEnabled" ), mDirCachingIsEnabled, false );
  addItem( itemDirCachingIsEnabled, TQString::fromLatin1( "DirCachingIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemDeleteMovesIntoTrashIsEnabled;
  itemDeleteMovesIntoTrashIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "DeleteMovesIntoTrashIsEnabled" ), mDeleteMovesIntoTrashIsEnabled, false );
  addItem( itemDeleteMovesIntoTrashIsEnabled, TQString::fromLatin1( "DeleteMovesIntoTrashIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemSkiplistIsEnabled;
  itemSkiplistIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "SkiplistIsEnabled" ), mSkiplistIsEnabled, false );
  addItem( itemSkiplistIsEnabled, TQString::fromLatin1( "SkiplistIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemPrioritylistIsEnabled;
  itemPrioritylistIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "PrioritylistIsEnabled" ), mPrioritylistIsEnabled, false );
  addItem( itemPrioritylistIsEnabled, TQString::fromLatin1( "PrioritylistIsEnabled" ) );
  TDEConfigSkeleton::ItemString  *itemOnQueueFinished;
  itemOnQueueFinished = new TDEConfigSkeleton::ItemString( currentGroup(), TQString::fromLatin1( "OnQueueFinished" ), mOnQueueFinished );
  addItem( itemOnQueueFinished, TQString::fromLatin1( "OnQueueFinished" ) );
  TDEConfigSkeleton::ItemBool  *itemOnQueueFinishedIsEnabled;
  itemOnQueueFinishedIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "OnQueueFinishedIsEnabled" ), mOnQueueFinishedIsEnabled, false );
  addItem( itemOnQueueFinishedIsEnabled, TQString::fromLatin1( "OnQueueFinishedIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemOnFileExistsOverwrite;
  itemOnFileExistsOverwrite = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "OnFileExistsOverwrite" ), mOnFileExistsOverwrite, false );
  addItem( itemOnFileExistsOverwrite, TQString::fromLatin1( "OnFileExistsOverwrite" ) );
  TDEConfigSkeleton::ItemBool  *itemOnFileExistsResume;
  itemOnFileExistsResume = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "OnFileExistsResume" ), mOnFileExistsResume, true );
  addItem( itemOnFileExistsResume, TQString::fromLatin1( "OnFileExistsResume" ) );
  TDEConfigSkeleton::ItemBool  *itemOnFileExistsSkip;
  itemOnFileExistsSkip = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "OnFileExistsSkip" ), mOnFileExistsSkip, false );
  addItem( itemOnFileExistsSkip, TQString::fromLatin1( "OnFileExistsSkip" ) );
  TDEConfigSkeleton::ItemBool  *itemOnFileExistsIsEnabled;
  itemOnFileExistsIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "OnFileExistsIsEnabled" ), mOnFileExistsIsEnabled, false );
  addItem( itemOnFileExistsIsEnabled, TQString::fromLatin1( "OnFileExistsIsEnabled" ) );

  setCurrentGroup( TQString::fromLatin1( "ui" ) );

  TDEConfigSkeleton::ItemBool  *itemSystrayIsEnabled;
  itemSystrayIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "SystrayIsEnabled" ), mSystrayIsEnabled, false );
  addItem( itemSystrayIsEnabled, TQString::fromLatin1( "SystrayIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemCommandLineIsEnabled;
  itemCommandLineIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "CommandLineIsEnabled" ), mCommandLineIsEnabled, true );
  addItem( itemCommandLineIsEnabled, TQString::fromLatin1( "CommandLineIsEnabled" ) );
  TDEConfigSkeleton::ItemBool  *itemHideHiddenFilesIsEnabled;
  itemHideHiddenFilesIsEnabled = new TDEConfigSkeleton::ItemBool( currentGroup(), TQString::fromLatin1( "HideHiddenFilesIsEnabled" ), mHideHiddenFilesIsEnabled, false );
  addItem( itemHideHiddenFilesIsEnabled, TQString::fromLatin1( "HideHiddenFilesIsEnabled" ) );
  TDEConfigSkeleton::ItemFont  *itemLogwindowFont;
  itemLogwindowFont = new TDEConfigSkeleton::ItemFont( currentGroup(), TQString::fromLatin1( "LogwindowFont" ), mLogwindowFont, TQFont( "DEC Terminal" ) );
  addItem( itemLogwindowFont, TQString::fromLatin1( "LogwindowFont" ) );

  setCurrentGroup( TQString::fromLatin1( "colors" ) );

  TDEConfigSkeleton::ItemColor  *itemLocalColor;
  itemLocalColor = new TDEConfigSkeleton::ItemColor( currentGroup(), TQString::fromLatin1( "LocalColor" ), mLocalColor, TQColor( 244,247,66 ) );
  addItem( itemLocalColor, TQString::fromLatin1( "LocalColor" ) );
  TDEConfigSkeleton::ItemColor  *itemSuccessColor;
  itemSuccessColor = new TDEConfigSkeleton::ItemColor( currentGroup(), TQString::fromLatin1( "SuccessColor" ), mSuccessColor, TQColor( 0,188,53 ) );
  addItem( itemSuccessColor, TQString::fromLatin1( "SuccessColor" ) );
  TDEConfigSkeleton::ItemColor  *itemBackgroundColor;
  itemBackgroundColor = new TDEConfigSkeleton::ItemColor( currentGroup(), TQString::fromLatin1( "BackgroundColor" ), mBackgroundColor, TQColor( 0,90,128 ) );
  addItem( itemBackgroundColor, TQString::fromLatin1( "BackgroundColor" ) );
  TDEConfigSkeleton::ItemColor  *itemFailureColor;
  itemFailureColor = new TDEConfigSkeleton::ItemColor( currentGroup(), TQString::fromLatin1( "FailureColor" ), mFailureColor, TQColor( 222,9,48 ) );
  addItem( itemFailureColor, TQString::fromLatin1( "FailureColor" ) );
}

KbConfig::~KbConfig()
{
  if ( mSelf == this )
    staticKbConfigDeleter.setObject( mSelf, 0, false );
}

