//
// C++ Interface: kbstatustip
//
// Description: 
//
//
// Author: Magnus Kulke <mkulke@magnusmachine>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef KBSTATUSTIP_H
#define KBSTATUSTIP_H

#include <tqtooltip.h>

/**
@author Magnus Kulke
*/
class KbStatusTip : public TQToolTip
{
public:
	KbStatusTip(TQWidget * widget);
	void maybeTip(const TQPoint &pos);
	void ShowStatus(TQString msg);
};

#endif
