//
// C++ Implementation: kbstatustip
//
// Description: 
//
//
// Author: Magnus Kulke <mkulke@magnusmachine>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <tqcursor.h>

#include "kbstatustip.h"

KbStatusTip::KbStatusTip(TQWidget * widget) : TQToolTip(widget)
{
}

void KbStatusTip::maybeTip(const TQPoint &)
{
	tip(parentWidget()->rect(), "kasablanca");
}

void KbStatusTip::ShowStatus(TQString msg)
{
	if ((TQCursor::pos().x() > parentWidget()->mapToGlobal(parentWidget()->pos()).x())
		&& (TQCursor::pos().y() > parentWidget()->mapToGlobal(parentWidget()->pos()).y())
		&& (TQCursor::pos().x() < (parentWidget()->mapToGlobal(parentWidget()->pos()).x() + parentWidget()->width()))
		&& (TQCursor::pos().y() < (parentWidget()->mapToGlobal(parentWidget()->pos()).y() + parentWidget()->height())))
	{
		tip(parentWidget()->rect(), msg);
	}
}
